@@ -1,15 +1,18 @@
 # copula_estimate.r ######################################################################################################
 # FUNCTION:               	DESCRIPTION:
 #  estimate.copula			Estimates the structure and the parameter of a HAC for a given sample.
-#  .ub         			 				Assures the dependency parameter of the initial node being smaller than parameter of consecutive nodes. (Internal function)
-#  . margins						Estimates the d marginal distributions for a d dimensional sample. (Internal function)   
-#  . one.mar						Estimates one marginal distributions for a given sample. (Internal function)   
-#  .max.min						0's contained in the data matrix are set to 0.000001 and 1's to 1-000001. (Internal function)
+#  .ub         			 	Assures the dependency parameter of the initial node being smaller than parameter of consecutive nodes. (Internal function)
+#  . margins				Estimates the d marginal distributions for a d dimensional sample. (Internal function)   
+#  . one.mar				Estimates one marginal distributions for a given sample. (Internal function)   
+#  .max.min					0's contained in the data matrix are set to 0.000001 and 1's to 1-000001. (Internal function)
+#  .constraints.ui          Returns the matrix for the constraints, i.e. ui of constrOptim.
+#  .constraints.ui          Returns the vector of lower bounds, i.e. ci of constrOptim.
+#  .rebuild                 Creates the tree of an hac-object for an ordered parameter vector.
 ##########################################################################################################################
 
-estimate.copula = function(X, type = HAC_GUMBEL, method = ML, epsilon = 0, agg.method = "mean", margins = NULL, theta.eps = 0.001, na.rm = FALSE, max.min = TRUE, ...){
+estimate.copula = function(X, type = HAC_GUMBEL, method = ML, hac = NULL, epsilon = 0, agg.method = "mean", margins = NULL, theta.eps = 0.001, na.rm = FALSE, max.min = TRUE, ...){
 	
-	if(is.null(colnames(X))){names = paste("X", 1 : NCOL(X), sep = "")}else names = colnames(X)
+	if(is.null(colnames(X))){g.names = names = paste("X", 1 : NCOL(X), sep = "")}else g.names = names = colnames(X)
 	
 	X = .margins(X, margins)
 	colnames(X) = names
@@ -22,9 +25,11 @@ estimate.copula = function(X, type = HAC_GUMBEL, method = ML, epsilon = 0, agg.m
 		
     if((method == TAU) && (dim(X)[1] == 1))stop("Cant estimate copula based on the tau method with just one observation")
     if(((type == HAC_GUMBEL) | (type == HAC_CLAYTON)) & (NCOL(X)>2)){
-        main.dim = NCOL(X)
-        tree = as.list(colnames(X))
-    	for(main.i in 1:(main.dim-2)){
+    
+        if(method != FML){
+           main.dim = NCOL(X); tree = as.list(names)
+    	
+        for(main.i in 1:(main.dim-2)){
     
             if(method == TAU){
             X.help = cor(X, method = "kendall")
@@ -36,12 +41,9 @@ estimate.copula = function(X, type = HAC_GUMBEL, method = ML, epsilon = 0, agg.m
                     matr[i, j] = matr[j, i] = tau2theta(optimise(f = function(y, i, j){sum(log(.dAC(X[,i], X[,j], tau2theta(y, type), type)))}, i = i, j = j, interval = c(0 + theta.eps, 1 - theta.eps), maximum = TRUE)$maximum, type)
             }
 
-            cur.dim = dim(matr)[1]
-            max.m = -10
-            max.i = max.ii = 0
-            for(i in 1:cur.dim)for(ii in i:cur.dim)if(i != ii){if(matr[i,ii] > max.m){max.m = matr[i,ii];max.i = i;max.ii = ii}} # what should be joined
+            cur.dim = NROW(matr); max.m = -10;max.i = max.ii = 0; sub.min = 1000
+            for(i in 1:cur.dim)for(ii in i:cur.dim)if(i != ii){if(matr[i,ii] > max.m){max.m = matr[i,ii];max.i = i;max.ii = ii}}
 
-            sub.min = 1000
             if(class(tree[[max.i]]) != "character") sub.min = c(sub.min, tree[[max.i]][[length(tree[[max.i]])]])
             if(class(tree[[max.ii]]) != "character") sub.min = c(sub.min, tree[[max.ii]][[length(tree[[max.ii]])]])
             if(min(min(sub.min), matr[max.i,max.ii]) == matr[max.i,max.ii]){sub.min = matr[max.i,max.ii]}else{sub.min = min(sub.min) - theta.eps}
@@ -52,29 +54,45 @@ estimate.copula = function(X, type = HAC_GUMBEL, method = ML, epsilon = 0, agg.m
 
             tree[[max.i]] = list(tree[[max.i]], tree[[max.ii]], max(sub.min, tau2theta(theta.eps, type)))
             tree = tree[-max.ii]; main.i = main.i+1
-        }
+            }
+        
         if(method == TAU){
             res = c(list(tree[[1]]), list(tree[[2]]), tau2theta(max(cor(X[,1], X[,2], method = "kendall"), theta.eps), type))
             res = .union(res, epsilon, method = agg.method, ...)
         }else{  			
             res = c(list(tree[[1]]), list(tree[[2]]), tau2theta(optimise(f = function(y, i, j){sum(log(.dAC(X[,1], X[,2], tau2theta(y, type), type)))}, i = 1, j = 2, interval = c(0 + theta.eps, .ub(tree[[1]][[length(tree[[1]])]], tree[[2]][[length(tree[[2]])]], type)), maximum = TRUE)$maximum, type))
             res = .union(res, epsilon, method = agg.method, ...)
-        }
-    }else if((type == AC_GUMBEL) | (type == HAC_GUMBEL)){
+    }}else{
+       if(is.null(hac)){
+          stop("A hac object is required.")
+       }else{
+          values = get.params(hac, sort.v = TRUE, decreasing=FALSE)
+          tree.full = hac$tree
+          n.params = length(values)
+          initial = numeric(n.params); initial[1]=if(type == HAC_GUMBEL){1}else{0}
+          constr = .constraints.ui(tree.full)
+          m = diag(1, nrow = constr+1, ncol = constr+1)
+          LL = to.logLik(X, hac)
+          optim = constrOptim(values, f=LL, grad=NULL, ui=m, ci=as.vector(.constraints.ci(tree.full, values, init = initial)), control=list(fnscale=-1), hessian=FALSE)
+          theta = optim$par
+          res = .rebuild(tree.full, values, theta)
+    }}}else{ 
+    
+    if((type == AC_GUMBEL) | (type == HAC_GUMBEL)){
         if(method == TAU){
             stop("No estimation based on taus for simple Archimedean copulas (with d>2, (d=2 not implemented yet))")
         }else{
-            res = list(names, theta = fitCopula(gumbelCopula(1.5, dim = dim(X)[2]), X, method = "ml")@estimate)
+            res = c(as.list(names), fitCopula(gumbelCopula(1.5, dim = dim(X)[2]), X, method = "ml")@estimate)
         }
     }else if((type == AC_CLAYTON) | (type == HAC_CLAYTON)){
         if(method == TAU){
             stop("No estimation based on taus for simple Archimedean copulas (with d>2, (d=2 not implemented yet))")
         }else{
-            res = list(names, theta = fitCopula(claytonCopula(1.5, dim = dim(X)[2]), X, method = "ml")@estimate)
+            res = c(as.list(names), fitCopula(claytonCopula(1.5, dim = dim(X)[2]), X, method = "ml")@estimate)
         }
     }else if(type == HAC_ROTATED_GUMBEL){
         stop("no yet implemented")
-    }
+    }}
 	hac(type = type, tree = res)
 }
 
@@ -148,3 +166,67 @@ estimate.copula = function(X, type = HAC_GUMBEL, method = ML, epsilon = 0, agg.m
 		else{
 			X}
 }
+
+#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
+
+.constraints.ui = function(tree){
+     if(length(tree)==1){tree=tree[[1]]}
+     n = length(tree)
+     s = sapply(tree[-n], is.character)
+ 
+     if(any(s==TRUE)){
+         if(any(s==FALSE)){
+            n.constr = length(which(!s))
+            n.constr = sum(n.constr, sapply(tree[which(!s)], .constraints.ui))
+         }else{
+            n.constr = 0
+         }}else{
+            n.constr = sum(length(which(!s)), sapply(tree[-n], .constraints.ui))
+        }
+    return(n.constr)
+}
+
+#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
+
+.constraints.ci = function(tree, values, init){
+     if(length(tree)==1){tree=tree[[1]]}
+     n = length(tree)
+     s = sapply(tree[-n], is.character)
+ 
+     if(any(s==TRUE)){
+         if(any(s==FALSE)){
+            n.constr = length(which(!s))
+            init[which(values %in% sort(sapply(tree[which(!s)], FUN=function(r){r[[length(r)]]})))] = rep(tree[[n]], n.constr)
+            
+            for(i in which(!s)){
+                init = .constraints.ci(tree[i], values, init)            
+            }
+         }}else{
+            n.constr = length(s)
+            init[which(values %in% sort(sapply(tree[-n], FUN=function(r){r[[length(r)]]})))] = rep(tree[[n]], n.constr)
+            
+            for(i in 1:(n-1)){
+                 init = .constraints.ci(tree[i], values, init)            
+            }
+        }
+    return(init)
+}
+
+#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
+
+.rebuild = function(tree, values, theta){
+     if(length(tree)==1){tree=tree[[1]]}
+     n = length(tree)
+     s = sapply(tree[-n], is.character)
+     tree[[n]]=theta[which(values==tree[[n]])]
+                 
+     if(any(s==TRUE)){
+         if(any(s==FALSE)){
+            tree=c(tree[which(s)], lapply(tree[which(!s)], .rebuild, values, theta), tree[[n]])           
+        }else{
+            tree=tree
+        }}else{
+            tree = c(lapply(tree[-n], .rebuild, values, theta), tree[[n]])
+        }
+    return(tree)
+}
